/*
 *  This file is part of "Proficio",
 *  a web-based self-learning applications framework.
 *
 *  Copyright (C) 2007-2009 Olivier Jacquemin.
 *
 *  "Proficio" is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  "Proficio" is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program: see the file COPYING.txt.  If not, see
 *  http://www.gnu.org/licenses/.
 */

/*extern QuestionsGenerator, IntegerOperationsGenerator, Multiplication,
  JsUnitException,
  assert, assertEquals, assertFalse, assertNotEquals, assertNotUndefined,
  debug, fail */

/**
 * @returns The names of all test functions in this page.
 */
function exposeTestFunctionNames() {
    return [
        "testCorrectInheritanceFromQuestionsGenerator",
        "testAvailableParameters",
        "testParameterTableDescription",
        "testParameterTableSetValueLetter",
        "testParameterTableSetValueStringConvertibleToNatural",
        "testGetMaxDifferentQuestions",
        "testGenerateQuestionsParameterNotSet",
        "testGenerateQuestionsZeroValueForParameter",
        "testGenerateQuestionsReturnsCorrectSize",
        "testGenerateQuestionsReturnsMultiplications",
        "testGenerateQuestionsTooFewRequested",
        "testGenerateQuestionsTooManyRequested",
        "testGenerateQuestionsMaxAllDifferent"
    ];
}

function testCorrectInheritanceFromQuestionsGenerator() {
    var multGen = new IntegerOperationsGenerator(Multiplication);
    var questGen = new QuestionsGenerator();

    assertNotUndefined("generateQuestions method", multGen.generateQuestions);
    assertEquals("Should not override generateQuestions method.",
		 questGen.generateQuestions, multGen.generateQuestions);
    assertNotUndefined("questionExists method", multGen.questionExists);
    assertEquals("Should not override questionExists method.",
		 questGen.questionExists, multGen.questionExists);

    assertNotUndefined("getMaxDifferentQuestions method", multGen.getMaxDifferentQuestions);
    assertNotEquals("Should override getMaxDifferentQuestions method.",
		    questGen.getMaxDifferentQuestions, multGen.getMaxDifferentQuestions);
    assertNotUndefined("generateNewQuestion method", multGen.generateNewQuestion);
    assertNotEquals("Should override generateNewQuestion method.",
                    questGen.generateNewQuestion, multGen.generateNewQuestion);
}

function testAvailableParameters() {
    var multGen = new IntegerOperationsGenerator(Multiplication);
    assertNotUndefined("Parameters list should be defined.",
                       multGen.parameters);
    assertNotUndefined("Parameter with ID 'table' should exist",
                       multGen.parameters.table);
}

function testParameterTableDescription() {
    var multGen = new IntegerOperationsGenerator(Multiplication);
    var paramTable = multGen.parameters.table;

    assertEquals("Incorrect description of table parameter.",
                 "Table de :", paramTable.description);
}

function testParameterTableSetValueLetter() {
    var multGen = new IntegerOperationsGenerator(Multiplication);
    var paramTable = multGen.parameters.table;
    try {
        paramTable.setValue("a");
        fail("Should fail to set value.");
    }
    catch (e) {
        if (e.constructor === JsUnitException) {
            throw e;
        }
        debug(e);
    }
}

function testParameterTableSetValueStringConvertibleToNatural() {
    var multGen = new IntegerOperationsGenerator(Multiplication);
    var paramTable = multGen.parameters.table;

    paramTable.setValue("3");
    assert("Value should have been converted into a number.",
           paramTable.value === 3);
}

function testGetMaxDifferentQuestions() {
    var multGen = new IntegerOperationsGenerator(Multiplication);
    multGen.parameters.table.setValue("7");
    multGen.parameters.lowerOperand.setValue("0");
    multGen.parameters.upperOperand.setValue("10");

    assertEquals(21, multGen.getMaxDifferentQuestions());
}

function testGenerateQuestionsParameterNotSet() {
    var multGen = new IntegerOperationsGenerator(Multiplication);

    // No parameter set.
    try {
        multGen.generateQuestions(1);
        fail("Should fail to generate questions.");
    }
    catch (e) {
        if (e.constructor === JsUnitException) {
            throw e;
        }
        debug(e);
    }

    // All but one parameter set: still not enough.
    multGen.parameters.table.setValue("7");
    multGen.parameters.lowerOperand.setValue("10");
    try {
        multGen.generateQuestions(1);
        fail("Should fail to generate questions.");
    }
    catch (e) {
        if (e.constructor === JsUnitException) {
            throw e;
        }
        debug(e);
    }
}

function testGenerateQuestionsZeroValueForParameter() {
    var multGen = new IntegerOperationsGenerator(Multiplication);
    multGen.parameters.table.setValue("0");
    multGen.parameters.lowerOperand.setValue("0");
    multGen.parameters.upperOperand.setValue("10");

    var questions = multGen.generateQuestions(2);
}

function testGenerateQuestionsReturnsCorrectSize() {
    var multGen = new IntegerOperationsGenerator(Multiplication);
    multGen.parameters.table.setValue("7");
    multGen.parameters.lowerOperand.setValue("0");
    multGen.parameters.upperOperand.setValue("10");

    var questions = multGen.generateQuestions(2);
    assertEquals("Number of generated questions",
                 2, questions.length);
}

function testGenerateQuestionsReturnsMultiplications() {
    var multGen = new IntegerOperationsGenerator(Multiplication);
    multGen.parameters.table.setValue("7");
    multGen.parameters.lowerOperand.setValue("0");
    multGen.parameters.upperOperand.setValue("10");

    var questions = multGen.generateQuestions(1);
    assertEquals("Type of questions generated",
                 Multiplication, questions[0].constructor);
}

function testGenerateQuestionsTooFewRequested() {
    var multGen = new IntegerOperationsGenerator(Multiplication);
    multGen.parameters.table.setValue("7");
    multGen.parameters.lowerOperand.setValue("0");
    multGen.parameters.upperOperand.setValue("10");

    try {
        multGen.generateQuestions(22);
        fail("Generating 22 questions should be impossible");
    }
    catch (e) {
        if (e.constructor === JsUnitException) {
            throw e;
        }
        debug(e);
    }
}

function testGenerateQuestionsTooManyRequested() {
    var multGen = new IntegerOperationsGenerator(Multiplication);
    multGen.parameters.table.setValue("7");
    multGen.parameters.lowerOperand.setValue("0");
    multGen.parameters.upperOperand.setValue("10");

    try {
        multGen.generateQuestions(-1);
        fail("Generating -1 question should be impossible");
    }
    catch (e) {
        if (e.constructor === JsUnitException) {
            throw e;
        }
    }
}

function testGenerateQuestionsMaxAllDifferent() {
    var multGen = new IntegerOperationsGenerator(Multiplication);
    multGen.parameters.table.setValue("7");
    multGen.parameters.lowerOperand.setValue("0");
    multGen.parameters.upperOperand.setValue("10");
    var maxDifferentQuestions = multGen.getMaxDifferentQuestions();

    var questions = multGen.generateQuestions(maxDifferentQuestions);
    for (var i = 0; i < maxDifferentQuestions; i += 1) {
        for (var j = 0; j < maxDifferentQuestions; j += 1) {
            if (i !== j) {
                assertFalse("Duplicate question", questions[i].equals(questions[j]));
            }
        }
    }
}

